/// <reference types="bn.js" />
import { PublicKey } from "@solana/web3.js";
import { TransactionBuilder } from "@orca-so/common-sdk";
import { ConfigOptionParams, MarketStateData, EscrowContext, ProfileData, PDA, OfferQuotes, EscrowFetcher } from "..";
import { BN } from "@project-serum/anchor";
export declare class MarketClient {
    ctx: EscrowContext;
    escrowFetcher: EscrowFetcher;
    publicKey: PublicKey;
    data: MarketStateData;
    pda: PDA;
    tokenMint: PublicKey;
    constructor(ctx: EscrowContext, publicKey: PublicKey, tokenMint: PublicKey, pda: PDA, data: MarketStateData);
    static getMarket(ctx: EscrowContext, configId: string, marketId: string, tokenMint: PublicKey): Promise<MarketClient>;
    getOfferBondAmount(side: any, user: PublicKey): Promise<{
        native: number;
        spl: number;
    }>;
    setTradeConfigAuthority(deployerPubkey: PublicKey, newTradeConfigAuthority: PublicKey): Promise<TransactionBuilder>;
    setAmlPolicyAuthority(deployerPubkey: PublicKey, newAmlPolicyAuthority: PublicKey): Promise<TransactionBuilder>;
    addArbitratorCert(tradeConfigAuthority: PublicKey, arbitratorAuthority: PublicKey): Promise<TransactionBuilder>;
    addTokenMetadata(tradeConfigAuthority: PublicKey, quota30Days: BN[], quota24Hours: BN[]): Promise<TransactionBuilder>;
    addTokenOracleMetadata(tradeConfigAuthority: PublicKey, renecOraclePubkey: PublicKey, quota30Days: BN[], quota24Hours: BN[]): Promise<TransactionBuilder>;
    setRenecRate(tradeConfigAuthority: PublicKey, renecRate: BN): Promise<TransactionBuilder>;
    setUsdRate(tradeConfigAuthority: PublicKey, usdRate: BN): Promise<TransactionBuilder>;
    setMinmumOfferSize(tradeConfigAuthority: PublicKey, minimumOfferSize: BN): Promise<TransactionBuilder>;
    removeArbitratorCert(tradeConfigAuthority: PublicKey, arbitratorAuthority: PublicKey): Promise<TransactionBuilder>;
    updateMarketState(tradeConfigAuthority: PublicKey, option: ConfigOptionParams): Promise<TransactionBuilder>;
    setProfile(amlPolicyAuthority: PublicKey, user: PublicKey, level: number): Promise<TransactionBuilder>;
    getProfile(user: PublicKey): Promise<ProfileData>;
    isArbitrator(arbitratorAuthority: PublicKey): Promise<boolean>;
    refresh(): Promise<MarketClient>;
    getOffersQuotesMatchingEngine(user: PublicKey, totalTradeAmount: BN, fiatCurrency: string, bondType?: object): Promise<OfferQuotes | null>;
    private buildMatchingEngineTransaction;
    private matchingEngine;
    /**
     * @param totalTradeAmount Raw token amount
     */
    matchingEngineWithExactCryptoAmount(user: PublicKey, totalTradeAmount: BN, offerQuotes: OfferQuotes, useBondless: boolean, takerBondType: object, buyOfferAccounts?: PublicKey[]): Promise<{
        txs: TransactionBuilder[];
        ids: BN[][];
    }>;
    /**
     * @param totalTradeAmount UI fiat amount
     */
    matchingEngineWithExactFiatAmount(user: PublicKey, totalTradeAmount: number, offerQuotes: OfferQuotes, useBondless: boolean, takerBondType: object, buyOfferAccounts?: PublicKey[]): Promise<{
        txs: TransactionBuilder[];
        ids: BN[][];
    }>;
    private requestCreateTrade;
    /**
     * @param tradeAmount Raw token amount
     */
    requestCreateTradeWithExactCryptoAmount(sellOfferId: BN, user: PublicKey, tradeAmount: BN, expectedRate: number, useBondless: boolean, takerBondType: object, buyOfferAccount?: PublicKey): Promise<{
        tx: TransactionBuilder;
        id: BN;
    }>;
    /**
     * @param tradeAmount UI fiat amount
     */
    requestCreateTradeWithExactFiatAmount(sellOfferId: BN, user: PublicKey, tradeAmount: number, expectedRate: number, useBondless: boolean, takerBondType: object, buyOfferAccount?: PublicKey): Promise<{
        tx: TransactionBuilder;
        id: BN;
    }>;
}
